import React from 'react';
import './App.css';
import Inner from "./inner";
import Movable from "./Movable"; // The default

class App extends React.Component {
    constructor(props) {
        super(props);
        this.ref = React.createRef();
        this.movable = React.createRef();
    }


    state = {
        mouseMove: null,
        mouseUp: null,
        dockList: [{id: -1, side: 'top', size: 0}, {id: -2, side: 'left', size: 0}, {id: -3, side: 'bottom', size: 0}, {id: -4, side: 'right', size: 0}],
        windowList: [1, 2, 3],
        tabs: {2: [2, 3]},
        showSuggestions: false,
        filledDock: false,
        highlightX: 0,
        highlightY: 0,
        highlightWidth: 300,
        highlightHeight: 300,
        fill: [],
        sizeX: document.documentElement.clientWidth,
        sizeY: document.documentElement.clientHeight
    };

    mouseMove = (e) => {
        e.persist();
        this.setState({mouseMove: e});
    };

    mouseUp = (e) => {
        e.persist();
        this.setState({mouseUp: e});
    };

    dock = (id, side, size) => {
      this.setState((state) => ({showSuggestions: state.fill.length || state.dockList.length - 3 === state.windowList.length ? false : id, dockList: [...state.dockList, {id, side, size}]}))
    };

    highlight = (x, y, width, height) => {
        this.setState({highlightX: x,
            highlightY: y,
            highlightWidth: width,
            highlightHeight: height})
    };

    addToTabs = (parentId, childId) => {
        this.setState((state) => {
            // const parent = Array.isArray(state.windowList[parentId]) ? state.windowList[parentId] : [state.windowList[parentId]];
            // return ({windowList: [...state.windowList.filter(x => [parentId, childId].includes(parent[0])), [...parent, childId]]})
        })
    };

    undock = (id) => {
      this.setState((state) => ({showSuggestions: false, filledDock: this.state.filledDock === id ? false : this.state.filledDock, dockList: state.dockList.filter(x => x.id !== id), windowList: [...this.state.windowList.filter(x => x !== id), id]}))
    };

    unblur = () => {
      this.setState({showSuggestions: false})
    };

    fill = (id) => {
        this.setState(state => ({fill: [...state.fill, id]}));
    };

    unfill = (id) => {
        this.setState(state => ({fill: state.fill.filter(x => x !== id), windowList: [...this.state.windowList.filter(x => x !== id), id]}))
    };

    resizeDock = (id, size) => {
      this.setState((state) => ({dockList: state.dockList.map(x => {
          if (x.id === id) {
              return {...x, size}
          }
          return x;
      })}))
    };

    focus = (id) => {
        this.setState((state) => ({windowList: [...state.windowList.filter(x => x !== id), id]}))
    };


    window = (x, index) => {
        return <Movable ref={this.movable} {...this.state} id={x} key={x} containerX={this.state.sizeX-6} containerY={this.state.sizeY-6} highlight={this.highlight} isFilled={this.state.fill.includes(x)} isVirtualFilled={this.state.filledDock === x} unblur={this.unblur} onblur={index === this.state.windowList.length - 1 && this.state.showSuggestions} blur={index !== this.state.windowList.length - 1 && this.state.showSuggestions} fill={this.fill} unfill={this.unfill} dock={this.dock} undock={this.undock} focus={this.focus} resizeDock={this.resizeDock}>{(this.state.tabs[x] || [x]).map(y => <Inner onMouseDown={e => e.stopPropagation()} text={y}/>)}</Movable>
    };

    render() {
        let docked = this.state.dockList.map(x => x.id);
        let undocked = this.state.windowList.filter(x => !docked.includes(x));
        let index = 0;
        while((index)*(index) < undocked.length) {
            index++;
        }
        let leftSize = 0;
        let rightSize = 0;
        let topSize = 0;
        let bottomSize = 0;
        for (let index in this.state.dockList) {
            let item = this.state.dockList[index];
            if (item.side === 'left') {
                leftSize += item.size;
            } else if (item.side === 'right') {
                rightSize += item.size;
            } else if (item.side === 'top') {
                topSize += item.size;
            } else if (item.side === 'bottom') {
                bottomSize += item.size;
            }
        }
        let space = this.state.sizeX;
        if (this.state.showSuggestions && this.state.dockList.filter(x => x.id === this.state.showSuggestions).length > 0) {
            let item = this.state.dockList.filter(x => x.id === this.state.showSuggestions)[0];
            space = item.size + (['left', 'right'].includes(item.side) ? (this.state.sizeX - leftSize - rightSize) : (this.state.sizeY - topSize - bottomSize));
        }
        return (
            <div className="App">
                <header className="App-header">
                    <div  onMouseMove={this.mouseMove} onMouseUp={this.mouseUp} onMouseLeave={this.mouseUp} style={{
                        position: 'relative',
                        width: this.state.sizeX,
                        height: this.state.sizeY,
                        borderColor: 'gray',
                        borderWeight: 4,
                        borderStyle: 'solid'
                    }}>
                        {this.state.highlightWidth && this.state.highlightHeight ? <div className={'highlight'} style={{
                        position: 'absolute',
                        left: this.state.highlightX,
                        top: this.state.highlightY,
                        backgroundColor: 'rgba(255,255,255, 0.2)',
                        width: ['top', 'bottom'].includes(this.state.dock) ? this.state.dockSize : this.state.highlightWidth,
                        height: ['right', 'left'].includes(this.state.dock) ? this.state.dockSize : this.state.highlightHeight,
                        }}/> : null}
                        {this.state.windowList.map((x, index) => this.window(x, index))}
                        {this.state.showSuggestions && <div className="flex-container" style={{display: 'flex', width: index*70, height: index*70, flexWrap: 'wrap', alignContent: 'flex-start', position: 'absolute', left: (leftSize  + this.state.sizeX - rightSize - index*70) / 2, top: (topSize  + this.state.sizeY - bottomSize - index*70) / 2}}>
                            {undocked.map(x => <div key='icons' style={{backgroundColor: 'white', width: 50, height: 50, margin: 10, cursor: 'pointer', borderColor: 'gray',
                                borderWeight: 4,
                                borderStyle: 'solid'}} onClick={
                                e => {
                                    let dockList = JSON.parse(JSON.stringify(this.state.dockList));
                                    dockList.filter(x => x.id === this.state.showSuggestions)[0].size = space / 2;
                                    this.setState(state => ({showSuggestions: false, fill: [...state.fill, x], filledDock: state.showSuggestions, windowList: [...state.windowList.filter(y => y !== x), x], dockList}))
                                }}>{x}</div>)}
                        </div>}
                    </div>
                </header>
            </div>
        );
    }
}

export default App;
