import React from 'react';
import './App.css';
import Inner from "./inner"; // The default
import Fullscreen from './fullscreen.svg';
import unFullscreen from './unfullscreen.svg';
import {Tabs} from "antd";
import 'antd/dist/antd.css';
const { TabPane } = Tabs;

class Movable extends React.Component {
    constructor(props) {
        super(props);
        this.ref = React.createRef();
        this.fullscreenRef = React.createRef();
    }


    state = {
        floating: false,
        resize: false,
        positionX: 0,
        positionY: 0,
        lastX: null,
        lastY: null,
        dock: null,
        sizeX: 300,
        sizeY: 300,
        minSizeX: 300,
        minSizeY: 300,
        prevSizeX: 300,
        prevSizeY: 300,
        cursor: 'auto',
    };

    mouseDown = (e) => {
        e.persist();
        let rect = this.ref.current.getBoundingClientRect();
        let inside;
        if (this.fullscreenRef.current) {
            let fullscreenRect = this.fullscreenRef.current.getBoundingClientRect();
            inside = fullscreenRect.x < e.pageX && fullscreenRect.x + fullscreenRect.width > e.pageX && fullscreenRect.y < e.pageY && fullscreenRect.y + fullscreenRect.height > e.pageY;
        }
        let offsetX = e.pageX - rect.x;
        let offsetY = e.pageY - rect.y;

        if (!this.props.isFilled) {
            if (offsetX < 10 && this.state.dock !== 'left') {
                e.preventDefault();
                this.setState((state) => ({resize: !state.resize ? ['left'] : state.resize.concat('left')}))
            }
            if (offsetY < 10 && this.state.dock !== 'top') {
                e.preventDefault();
                this.setState((state) => ({resize: !state.resize ? ['top'] : state.resize.concat('top')}))
            }
            if (offsetX > rect.width - 10 && this.state.dock !== 'right') {
                e.preventDefault();
                this.setState((state) => ({resize: !state.resize ? ['right'] : state.resize.concat('right')}))
            }
            if (offsetY > rect.height - 10 && this.state.dock !== 'bottom') {
                e.preventDefault();
                this.setState((state) => ({resize: !state.resize ? ['bottom'] : state.resize.concat('bottom')}))
            } else if (offsetX >= 10 && offsetY >= 10 && offsetX <= rect.width - 10 && offsetY <= rect.height - 10 && !inside && !this.props.isVirtualFilled) {
                e.preventDefault();
                this.props.undock(this.props.id);
                this.props.focus(this.props.id);
                this.setState({
                    floating: true,
                    lastY: e.screenY,
                    lastX: e.screenX,
                    positionX: this.state.dock ? Math.min(Math.max(this.state.positionX + (e.pageX - rect.x) - 150, 0), this.props.containerX - this.state.sizeX) : this.state.positionX,
                    positionY: this.state.dock ? Math.min(Math.max(this.state.positionY + (e.pageY - rect.y) - 150, 0), this.props.containerY - this.state.sizeY) : this.state.positionY,
                    dock: null
                });
            }
        }
    };
    mouseUp = () => {
        if (this.state.resize) {
            this.setState({resize: null});
        } else if (this.state.floating) {
            let leftSize = 0;
            let rightSize = 0;
            let topSize = 0;
            let bottomSize = 0;
            for (let index in this.props.dockList) {
                let item = this.props.dockList[index];
                if (item.side === 'left') {
                    leftSize += item.size;
                    if (this.state.positionX < 40 + leftSize && this.state.positionX >= leftSize) {
                        for (let subindex = parseInt(index) + 1; subindex < this.props.dockList.length; subindex++) {
                            let subItem = this.props.dockList[subindex];
                            if (subItem.side === 'left') {
                                break;
                            } else if (subItem.side === 'right') {
                                rightSize += subItem.size;
                            } else if (subItem.side === 'top') {
                                topSize += subItem.size;
                            } else if (subItem.side === 'bottom') {
                                bottomSize += subItem.size;
                            }
                        }
                        this.props.dock(this.props.id, 'left', this.state.sizeX);
                        this.props.highlight(0, 0, 0, 0);
                        this.setState({
                            dock: 'left',
                            floating: false,
                            positionY: topSize,
                            positionX: leftSize,
                            dockSize: this.props.containerY - topSize - bottomSize
                        });
                        break;
                    }
                } else if (item.side === 'right') {
                    rightSize += item.size;
                    if (this.state.positionX > this.props.containerX - this.state.sizeX - 40 - rightSize && this.state.positionX <= this.props.containerX - this.state.sizeX - rightSize) {
                        for (let subindex = parseInt(index) + 1; subindex < this.props.dockList.length; subindex++) {
                            let subItem = this.props.dockList[subindex];
                            if (subItem.side === 'right') {
                                break;
                            } else if (subItem.side === 'left') {
                                leftSize += subItem.size;
                            } else if (subItem.side === 'top') {
                                topSize += subItem.size;
                            } else if (subItem.side === 'bottom') {
                                bottomSize += subItem.size;
                            }
                        }
                        this.props.dock(this.props.id, 'right', this.state.sizeX);
                        this.setState({
                            dock: 'right',
                            floating: false,
                            positionY: topSize,
                            positionX: this.props.containerX - this.state.sizeX - rightSize,
                            dockSize: this.props.containerY - topSize - bottomSize
                        });
                        break;
                    }
                } else if (item.side === 'top') {
                    topSize += item.size;
                    if (this.state.positionY < 40 + topSize && this.state.positionY >= topSize) {
                        for (let subindex = parseInt(index) + 1; subindex < this.props.dockList.length; subindex++) {
                            let subItem = this.props.dockList[subindex];
                            if (subItem.side === 'top') {
                                break;
                            } else if (subItem.side === 'left') {
                                leftSize += subItem.size;
                            } else if (subItem.side === 'right') {
                                rightSize += subItem.size;
                            } else if (subItem.side === 'bottom') {
                                bottomSize += subItem.size;
                            }
                        }
                        this.props.dock(this.props.id, 'top', this.state.sizeY);
                        this.setState({
                            dock: 'top',
                            floating: false,
                            positionX: leftSize,
                            positionY: topSize,
                            dockSize: this.props.containerX - rightSize - leftSize
                        });
                        break;
                    }
                } else if (item.side === 'bottom') {
                    bottomSize += item.size;
                    if (this.state.positionY > this.props.containerY - this.state.sizeY - 40 - bottomSize && this.state.positionY <= this.props.containerY - this.state.sizeY - bottomSize) {
                        for (let subindex = parseInt(index) + 1; subindex < this.props.dockList.length; subindex++) {
                            let subItem = this.props.dockList[subindex];
                            if (subItem.side === 'bottom') {
                                break;
                            } else if (subItem.side === 'left') {
                                leftSize += subItem.size;
                            } else if (subItem.side === 'right') {
                                rightSize += subItem.size;
                            } else if (subItem.side === 'top') {
                                topSize += subItem.size;
                            }
                        }
                        this.props.dock(this.props.id, 'bottom', this.state.sizeY);
                        this.setState({
                            dock: 'bottom',
                            floating: false,
                            positionX: leftSize,
                            positionY: this.props.containerY - bottomSize - this.state.sizeY,
                            dockSize: this.props.containerX - rightSize - leftSize
                        });
                        break;
                    }
                }
            }
            this.setState({floating: false})
        }
    };

    fill = () => {
        let leftSize = 0;
        let rightSize = 0;
        let topSize = 0;
        let bottomSize = 0;
        for (let index in this.props.dockList) {
            let item = this.props.dockList[index];
            if (item.side === 'left') {
                leftSize += item.size;
            } else if (item.side === 'right') {
                rightSize += item.size;
            } else if (item.side === 'top') {
                topSize += item.size;
            } else if (item.side === 'bottom') {
                bottomSize += item.size;
            }
        }
        this.setState({
            // fill: true,
            positionX: leftSize,
            positionY: topSize,
            sizeX: this.props.containerX - leftSize - rightSize,
            sizeY: this.props.containerY - topSize - bottomSize,
            prevSizeX: this.state.sizeX,
            prevSizeY: this.state.sizeY,
        });
        this.props.focus(this.props.id);
        this.props.fill(this.props.id);
    };

    unfill = (e) => {
        let rect = this.ref.current.getBoundingClientRect();
        let leftSize = 0;
        let rightSize = 0;
        let topSize = 0;
        let bottomSize = 0;
        for (let index in this.props.dockList) {
            let item = this.props.dockList[index];
            if (item.side === 'left') {
                leftSize += item.size;
            } else if (item.side === 'right') {
                rightSize += item.size;
            } else if (item.side === 'top') {
                topSize += item.size;
            } else if (item.side === 'bottom') {
                bottomSize += item.size;
            }
        }
        this.setState({
            // fill: false,
            sizeX: this.state.prevSizeX,
            sizeY: this.state.prevSizeY,
            positionX: this.state.dock ? Math.min(Math.max(this.state.positionX + (e.pageX || 0 - rect.x) - 150, 0), this.props.containerX - this.state.sizeX) : this.state.positionX,
            positionY: this.state.dock ? Math.min(Math.max(this.state.positionY + (e.pageY || 0 - rect.y) - 150, 0), this.props.containerX - this.state.sizeY) : this.state.positionY,

        })
        this.props.unfill(this.props.id);
    };

    update = () => {
        const size = this.state.dock && this.props.dockList.filter(x => x.id === this.props.id)[0].size;
        if (this.state.dock && size !== (['left', 'right'].includes(this.state.dock) ? this.state.sizeX : this.state.sizeY)) {
            if (this.state.dock === 'left') {
                this.setState({sizeX: size});
            } else if (this.state.dock === 'right') {
                this.setState({sizeX: size, positionX: this.state.positionX - size + this.state.sizeX, prevSizeX: this.state.sizeX});
            } else if (this.state.dock === 'bottom') {
                this.setState({sizeY: size, positionY: this.state.positionY - size + this.state.sizeY, prevSizeY: this.state.sizeY});
            } else {
                this.setState({sizeY: size});
            }
        } else if (this.props.isFilled) {
            let leftSize = 0;
            let rightSize = 0;
            let topSize = 0;
            let bottomSize = 0;
            for (let index in this.props.dockList) {
                let item = this.props.dockList[index];
                if (item.side === 'left') {
                    leftSize += item.size;
                } else if (item.side === 'right') {
                    rightSize += item.size;
                } else if (item.side === 'top') {
                    topSize += item.size;
                } else if (item.side === 'bottom') {
                    bottomSize += item.size;
                }
            }
            this.setState({
                positionX: leftSize,
                positionY: topSize,
                sizeX: this.props.containerX - leftSize - rightSize,
                sizeY: this.props.containerY - topSize - bottomSize,
            })
        } else if (this.state.dock || this.state.resize) {
            let leftSize = 0;
            let rightSize = 0;
            let topSize = 0;
            let bottomSize = 0;
            for (let index in this.props.dockList) {
                let item = this.props.dockList[index];
                if (item.side === 'left') {
                    if (item.id === this.props.id) {
                        this.setState({
                            dock: 'left',
                            floating: false,
                            positionY: topSize,
                            positionX: leftSize,
                            dockSize: this.props.containerY - topSize - bottomSize
                        });
                        break;
                    }
                    leftSize += item.size;
                } else if (item.side === 'right') {
                    if (item.id === this.props.id) {
                        this.setState({
                            dock: 'right',
                            floating: false,
                            positionY: topSize,
                            positionX: this.props.containerX - this.state.sizeX - rightSize,
                            dockSize: this.props.containerY - topSize - bottomSize
                        });
                        break;
                    }
                    rightSize += item.size;
                } else if (item.side === 'top') {
                    if (item.id === this.props.id) {
                        this.setState({
                            dock: 'top',
                            floating: false,
                            positionX: leftSize,
                            positionY: topSize,
                            dockSize: this.props.containerX - rightSize - leftSize
                        });
                        break;
                    }
                    topSize += item.size;
                } else if (item.side === 'bottom') {
                    if (item.id === this.props.id) {
                        this.setState({
                            dock: 'bottom',
                            floating: false,
                            positionX: leftSize,
                            positionY: this.props.containerY - bottomSize - this.state.sizeY,
                            dockSize: this.props.containerX - rightSize - leftSize
                        });
                        break;
                    }
                    bottomSize += item.size;
                }
            }
        }
    };

    mouseMove = (e) => {
        let cursor = this.props.isFilled || this.props.isVirtualFilled ? 'auto' : 'pointer';
        if (!this.state.floating && !this.props.isFilled) {
            let rect = this.ref.current.getBoundingClientRect();
            let offsetX = e.pageX - rect.x;
            let offsetY = e.pageY - rect.y;
            if (((offsetY < 10 && offsetX > rect.width - 10) || (offsetX < 10 && offsetY > rect.height - 10)) && !this.state.dock) {
                cursor = 'ne-resize';
            } else if (((offsetY < 10 && offsetX < 10) || (offsetX > rect.width - 10 && offsetY > rect.height - 10)) && !this.state.dock) {
                cursor = 'nw-resize';
            } else if ((offsetX < 10 && (this.state.dock === 'right' || !this.state.dock)) || (offsetX > rect.width - 10 && (this.state.dock === 'left' || !this.state.dock))) {
                cursor = 'e-resize';
            } else if ((offsetY < 10 && (this.state.dock === 'bottom' || !this.state.dock)) || (offsetY > rect.height - 10 && (this.state.dock === 'top' || !this.state.dock))) {
                cursor = 'n-resize';
            }
        }

        e.stopPropagation();
        if (this.state.floating && !this.props.isFilled) {
            ////
            let leftSize = 0;
            let rightSize = 0;
            let topSize = 0;
            let bottomSize = 0;
            let side = '';
            let highlightX = 0;
            let highlightY = 0;
            let highlightWidth = 0;
            let highlightHeight = 0;
            for (let index in this.props.dockList) {
                let item = this.props.dockList[index];
                if (item.side === 'left') {
                    leftSize += item.size;
                    if (this.state.positionX < 40 + leftSize && this.state.positionX >= leftSize) {
                        side = 'left';
                    }
                } else if (item.side === 'right') {
                    rightSize += item.size;
                    if (this.state.positionX > this.props.containerX - this.state.sizeX - 40 - rightSize && this.state.positionX <= this.props.containerX - this.state.sizeX - rightSize) {
                        side = 'right';
                    }
                } else if (item.side === 'top') {
                    topSize += item.size;
                    if (this.state.positionY < 40 + topSize && this.state.positionY >= topSize) {
                        side = 'top';
                    }
                } else if (item.side === 'bottom') {
                    bottomSize += item.size;
                    if (this.state.positionY > this.props.containerY - this.state.sizeY - 40 - bottomSize && this.state.positionY <= this.props.containerY - this.state.sizeY - bottomSize) {
                        side = 'bottom';
                    }
                }
            }
            if (side === 'left') {
                highlightX = leftSize;
                highlightY = topSize;
                highlightWidth = this.state.sizeX;
                highlightHeight = this.props.containerY - topSize - bottomSize;
            } else if (side === 'right') {
                highlightX = this.props.containerX - rightSize - this.state.sizeX;
                highlightY = topSize;
                highlightWidth = this.state.sizeX;
                highlightHeight = this.props.containerY - topSize - bottomSize;
            } else if (side === 'top') {
                highlightX = leftSize;
                highlightY = topSize;
                highlightWidth = this.props.containerX - leftSize - rightSize;
                highlightHeight = this.state.sizeY;
            } else if (side === 'bottom') {
                highlightX = leftSize;
                highlightY = this.props.containerY - this.state.sizeY - bottomSize;
                highlightWidth = this.props.containerX - leftSize - rightSize;
                highlightHeight = this.state.sizeY;
            }

            ////
            e.persist();
            let movementX = this.state.lastX != null ? (e.screenX - this.state.lastX) : 0;
            let movementY = this.state.lastY != null ? (e.screenY - this.state.lastY) : 0;
            this.setState((state) => ({
                ...state,
                positionX: Math.min(Math.max(state.positionX + movementX, 0), this.props.containerX - this.state.sizeX),
                positionY: Math.min(Math.max(state.positionY + movementY, 0), this.props.containerY - this.state.sizeY),
                lastX: e.screenX,
                lastY: e.screenY,
                cursor
            }));
            this.props.highlight(highlightX, highlightY, highlightWidth, highlightHeight)

        } else if (this.state.resize) {
            let rect = this.ref.current.getBoundingClientRect();
            let offsetX = e.pageX - rect.x;
            let offsetY = e.pageY - rect.y;
            let sizeX = Math.max(this.state.minSizeX, this.state.resize.includes('right') ? offsetX : this.state.resize.includes('left') ? this.state.sizeX - offsetX : this.state.sizeX);
            let sizeY = Math.max(this.state.minSizeY, this.state.resize.includes('bottom') ? offsetY : this.state.resize.includes('top') ? this.state.sizeY - offsetY : this.state.sizeY);
            this.props.resizeDock(this.props.id, ['top', 'bottom'].includes(this.state.dock) ? sizeY : sizeX);
            this.setState(state => ({
                sizeX,
                sizeY,
                positionX: state.resize.includes('left') ? state.positionX + state.sizeX - sizeX : state.positionX,
                positionY: state.resize.includes('top') ? state.positionY + state.sizeY - sizeY : state.positionY,
                lastX: e.screenX,
                lastY: e.screenY,
                cursor
            }))
        } else {
            let rect = this.ref.current.getBoundingClientRect();
            let offsetX = e.pageX - rect.x;
            let offsetY = e.pageY - rect.y;
            if (offsetY > 0 && (offsetY < this.state.sizeY || ['left', 'right'].includes(this.state.dock)) && offsetX > 0 && (offsetX < this.state.sizeX || ['top', 'bottom'].includes(this.state.dock))) {
                this.setState({cursor});
            }
        }
    };


    equals = function (src, array) {
        // if the other array is a falsy value, return
        if (!array)
            return false;

        // compare lengths - can save a lot of time
        if (src.length !== array.length)
            return false;

        for (var i = 0, l = src.length; i < l; i++) {
            // Check if we have nested arrays
            if (src[i] instanceof Array && array[i] instanceof Array) {
                // recurse into the nested arrays
                if (!src[i].equals(array[i]))
                    return false;
            } else if (src[i] !== array[i]) {
                // Warning - two different object instances will never be equal: {x:20} != {x:20}
                return false;
            }
        }
        return true;
    };

    componentDidUpdate(prevProps, prevState, snapshot) {
        if (prevProps.mouseMove !== this.props.mouseMove) {
            this.mouseMove(this.props.mouseMove)
        }
        if (prevProps.mouseUp !== this.props.mouseUp) {
            this.mouseUp(this.props.mouseUp)
        }
        if ((!this.equals(prevProps.dockList, this.props.dockList) && (this.state.dock || this.props.isFilled)) || prevProps.isFilled !== this.props.isFilled) {
            this.update()
        }
    }

    render() {
        return [
            this.props.onblur && <div key='blur' onClick={e => this.props.unblur()} style={{position: 'absolute', left: 0, top: 0, width: '100%', height: '100%', backgroundColor: 'rgba(0,0,0,0.3)'}}/>,
            <div onMouseDown={this.mouseDown} ref={this.ref} style={{
                position: 'absolute',
                left: this.state.positionX,
                top: this.state.positionY,
                boxShadow: '0px 3px 10px rgba(0, 0, 0, 0.3)',
                backgroundColor: 'white',
                width: ['top', 'bottom'].includes(this.state.dock) ? this.state.dockSize : this.state.sizeX,
                height: ['right', 'left'].includes(this.state.dock) ? this.state.dockSize : this.state.sizeY,
                filter: this.props.blur && !this.state.dock? 'blur(3px)' : ''
            }}>
                { !this.state.floating && !this.props.isFilled && <>
                { !this.state.dock && <div style={{position: 'absolute', top: 0, left: 0, width: 10, height: 10, cursor: 'nw-resize'}}/>}
                { !this.state.dock && <div style={{position: 'absolute', top: this.state.sizeY - 10, left: this.state.sizeX - 10, width: 10, height: 10, cursor: 'nw-resize'}}/>}
                { !this.state.dock && <div style={{position: 'absolute', top:  this.state.sizeY - 10, left: 0, width: 10, height: 10, cursor: 'ne-resize'}}/>}
                { !this.state.dock && <div style={{position: 'absolute', top: 0, left: this.state.sizeX - 10, width: 10, height: 10, cursor: 'ne-resize'}}/>}
                { (!this.state.dock || this.state.dock === 'right') && <div style={{position: 'absolute', top: 10, left: 0, width: 10, height: ['left', 'right'].includes(this.state.dock) ? this.state.dockSize - 20 : this.state.sizeX - 20, cursor: 'e-resize'}}/>}
                { (!this.state.dock || this.state.dock === 'left') && <div style={{position: 'absolute', top: 10, left: ['top', 'bottom'].includes(this.state.dock) ? this.state.dockSize - 10 : this.state.sizeX -10, width: 10, height: ['left', 'right'].includes(this.state.dock) ? this.state.dockSize - 20 : this.state.sizeX - 20, cursor: 'e-resize'}}/>}
                { (!this.state.dock || this.state.dock === 'bottom') && <div style={{position: 'absolute', left: 10, top: 0, height: 10, width: ['top', 'bottom'].includes(this.state.dock) ? this.state.dockSize - 20 : this.state.sizeY - 20, cursor: 'n-resize'}}/>}
                { (!this.state.dock || this.state.dock === 'top') && <div style={{position: 'absolute', left: 10, top: ['left', 'right'].includes(this.state.dock) ? this.state.dockSize - 10 : this.state.sizeY -10, height: 10, width: ['top', 'bottom'].includes(this.state.dock) ? this.state.dockSize - 20 : this.state.sizeY - 20, cursor: 'n-resize'}}/>}
                </>}

                <div style={{position: 'absolute', top: 10, left: 10, width: ['top', 'bottom'].includes(this.state.dock) ? this.state.dockSize - 20 : this.state.sizeX -20, height: ['left', 'right'].includes(this.state.dock) ? this.state.dockSize - 20 : this.state.sizeY - 20 , cursor: this.props.isFilled ||  this.props.isVirtualFilled ? 'auto' : 'pointer'}}>
                    <Tabs defaultActiveKey="0">
                        {React.Children.toArray(this.props.children).map((x, index) => <TabPane tab={index} key={index}>{x}</TabPane>)}
                    </Tabs>
                </div>

                {(this.props.isVirtualFilled || !this.state.dock) && <span style={{position: 'absolute', top: 0, left: 0, cursor: 'grab', userSelect: 'none'}} ref={this.fullscreenRef}
                      onClick={e => {
                          if (this.state.dock) {
                              let rect = this.ref.current.getBoundingClientRect();
                              this.setState({
                                  positionX: Math.min(Math.max(this.state.positionX + (e.pageX - rect.x) - 150, 0), this.props.containerX - this.state.sizeX),
                                  positionY: Math.min(Math.max(this.state.positionY + (e.pageY - rect.y) - 150, 0), this.props.containerY - this.state.sizeY),
                                  sizeX: this.state.prevSizeX,
                                  sizeY: this.state.prevSizeY,
                                  dock: null
                              });
                              this.props.undock(this.props.id)
                          } else if (this.props.isFilled) {
                              this.unfill(e)
                          } else {
                              this.fill()
                          }
                      }}>
                    {(this.props.isFilled || this.state.dock) ? <img src={unFullscreen} width={30} height={30}/> :
                        <img src={Fullscreen} width={30} height={30}/>}
                </span>}
            </div>,
        ];
    }
}

export default Movable;